////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Copyright (c) Qualcomm Technologies, Inc. and/or its subsidiaries.
// All Rights Reserved.
// Confidential and Proprietary - Qualcomm Technologies, Inc.
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// THIS IS AUTO-GENERATED CODE.  PLEASE DO NOT EDIT (File bug reports against tools).
///
/// Auto-generated by: Parameter Parser V5.5.1 (2411131018)
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// @file  abf_4_0_0.cpp
/// @brief Auto-generated Chromatix parameter file
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include "abf_4_0_0.h"
#include "parametersetmanager.h"

ABF_4_0_0_NAMESPACE_BEGIN

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// SetValue/GetValue Functions
/// Gets or sets the requested value
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool abf40_rgn_dataType::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "minmax_maxshft") == 0)
    {
        ParameterUtils::DoubleToString((double)minmax_maxshft, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "minmax_minshft") == 0)
    {
        ParameterUtils::DoubleToString((double)minmax_minshft, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "minmax_offset") == 0)
    {
        ParameterUtils::DoubleToString((double)minmax_offset, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "minmax_bls") == 0)
    {
        ParameterUtils::DoubleToString((double)minmax_bls, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "blkpix_lev_tab") == 0)
    {
        __result = blkpix_lev_tab.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "noise_std_lut_tab") == 0)
    {
        __result = noise_std_lut_tab.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "curve_offset_tab") == 0)
    {
        __result = curve_offset_tab.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "dist_ker_tab") == 0)
    {
        __result = dist_ker_tab.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "edge_softness_tab") == 0)
    {
        __result = edge_softness_tab.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "denoise_strength_tab") == 0)
    {
        __result = denoise_strength_tab.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "noise_prsv_anchor_tab") == 0)
    {
        __result = noise_prsv_anchor_tab.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "noise_prsv_base_tab") == 0)
    {
        __result = noise_prsv_base_tab.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "radial_edge_softness_adj_tab") == 0)
    {
        __result = radial_edge_softness_adj_tab.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "radial_noise_prsv_adj_tab") == 0)
    {
        __result = radial_noise_prsv_adj_tab.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "act_fac_lut_tab") == 0)
    {
        __result = act_fac_lut_tab.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "act_fac0") == 0)
    {
        ParameterUtils::DoubleToString((double)act_fac0, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "act_fac1") == 0)
    {
        ParameterUtils::DoubleToString((double)act_fac1, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "act_thd0") == 0)
    {
        ParameterUtils::DoubleToString((double)act_thd0, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "act_thd1") == 0)
    {
        ParameterUtils::DoubleToString((double)act_thd1, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "act_smth_thd0") == 0)
    {
        ParameterUtils::DoubleToString((double)act_smth_thd0, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "act_smth_thd1") == 0)
    {
        ParameterUtils::DoubleToString((double)act_smth_thd1, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "dark_thd") == 0)
    {
        ParameterUtils::DoubleToString((double)dark_thd, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "dark_fac_lut_tab") == 0)
    {
        __result = dark_fac_lut_tab.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "edge_detect_thd") == 0)
    {
        ParameterUtils::DoubleToString((double)edge_detect_thd, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "edge_count_low") == 0)
    {
        ParameterUtils::DoubleToString((double)edge_count_low, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "edge_detect_noise_scaler") == 0)
    {
        ParameterUtils::DoubleToString((double)edge_detect_noise_scaler, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "edge_smooth_strength") == 0)
    {
        ParameterUtils::DoubleToString((double)edge_smooth_strength, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "edge_smooth_scaler_tab") == 0)
    {
        __result = edge_smooth_scaler_tab.GetValue(&__path[__pos], __value, __size);
    }

    return __result;
}

bool abf40_rgn_dataType::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "minmax_maxshft") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            minmax_maxshft = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "minmax_minshft") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            minmax_minshft = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "minmax_offset") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            minmax_offset = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "minmax_bls") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            minmax_bls = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "blkpix_lev_tab") == 0)
    {
        __result = blkpix_lev_tab.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "noise_std_lut_tab") == 0)
    {
        __result = noise_std_lut_tab.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "curve_offset_tab") == 0)
    {
        __result = curve_offset_tab.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "dist_ker_tab") == 0)
    {
        __result = dist_ker_tab.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "edge_softness_tab") == 0)
    {
        __result = edge_softness_tab.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "denoise_strength_tab") == 0)
    {
        __result = denoise_strength_tab.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "noise_prsv_anchor_tab") == 0)
    {
        __result = noise_prsv_anchor_tab.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "noise_prsv_base_tab") == 0)
    {
        __result = noise_prsv_base_tab.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "radial_edge_softness_adj_tab") == 0)
    {
        __result = radial_edge_softness_adj_tab.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "radial_noise_prsv_adj_tab") == 0)
    {
        __result = radial_noise_prsv_adj_tab.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "act_fac_lut_tab") == 0)
    {
        __result = act_fac_lut_tab.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "act_fac0") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            act_fac0 = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "act_fac1") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            act_fac1 = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "act_thd0") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            act_thd0 = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "act_thd1") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            act_thd1 = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "act_smth_thd0") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            act_smth_thd0 = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "act_smth_thd1") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            act_smth_thd1 = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "dark_thd") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            dark_thd = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "dark_fac_lut_tab") == 0)
    {
        __result = dark_fac_lut_tab.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "edge_detect_thd") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            edge_detect_thd = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "edge_count_low") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            edge_count_low = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "edge_detect_noise_scaler") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            edge_detect_noise_scaler = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "edge_smooth_strength") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            edge_smooth_strength = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "edge_smooth_scaler_tab") == 0)
    {
        __result = edge_smooth_scaler_tab.SetValue(&__path[__pos], __value);
    }

    return __result;
}

bool abf40_rgn_dataType::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // minmax_maxshft
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("minmax_maxshft", ParameterParserDataType::UInt32, (double)minmax_maxshft, 0));
    }
    else if (PARAMETER_STRCMP(__name, "minmax_maxshft") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::UInt32, (double)minmax_maxshft, 0));
    }

    // minmax_minshft
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("minmax_minshft", ParameterParserDataType::UInt32, (double)minmax_minshft, 0));
    }
    else if (PARAMETER_STRCMP(__name, "minmax_minshft") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::UInt32, (double)minmax_minshft, 0));
    }

    // minmax_offset
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("minmax_offset", ParameterParserDataType::UInt32, (double)minmax_offset, 0));
    }
    else if (PARAMETER_STRCMP(__name, "minmax_offset") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::UInt32, (double)minmax_offset, 0));
    }

    // minmax_bls
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("minmax_bls", ParameterParserDataType::Int32, (double)minmax_bls, 0));
    }
    else if (PARAMETER_STRCMP(__name, "minmax_bls") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Int32, (double)minmax_bls, 0));
    }

    // blkpix_lev_tab
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("blkpix_lev_tab", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "blkpix_lev_tab") == 0)
    {
        __result = blkpix_lev_tab.GetInfo(&__path[__pos], __info);
    }

    // noise_std_lut_tab
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("noise_std_lut_tab", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "noise_std_lut_tab") == 0)
    {
        __result = noise_std_lut_tab.GetInfo(&__path[__pos], __info);
    }

    // curve_offset_tab
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("curve_offset_tab", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "curve_offset_tab") == 0)
    {
        __result = curve_offset_tab.GetInfo(&__path[__pos], __info);
    }

    // dist_ker_tab
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("dist_ker_tab", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "dist_ker_tab") == 0)
    {
        __result = dist_ker_tab.GetInfo(&__path[__pos], __info);
    }

    // edge_softness_tab
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("edge_softness_tab", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "edge_softness_tab") == 0)
    {
        __result = edge_softness_tab.GetInfo(&__path[__pos], __info);
    }

    // denoise_strength_tab
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("denoise_strength_tab", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "denoise_strength_tab") == 0)
    {
        __result = denoise_strength_tab.GetInfo(&__path[__pos], __info);
    }

    // noise_prsv_anchor_tab
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("noise_prsv_anchor_tab", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "noise_prsv_anchor_tab") == 0)
    {
        __result = noise_prsv_anchor_tab.GetInfo(&__path[__pos], __info);
    }

    // noise_prsv_base_tab
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("noise_prsv_base_tab", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "noise_prsv_base_tab") == 0)
    {
        __result = noise_prsv_base_tab.GetInfo(&__path[__pos], __info);
    }

    // radial_edge_softness_adj_tab
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("radial_edge_softness_adj_tab", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "radial_edge_softness_adj_tab") == 0)
    {
        __result = radial_edge_softness_adj_tab.GetInfo(&__path[__pos], __info);
    }

    // radial_noise_prsv_adj_tab
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("radial_noise_prsv_adj_tab", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "radial_noise_prsv_adj_tab") == 0)
    {
        __result = radial_noise_prsv_adj_tab.GetInfo(&__path[__pos], __info);
    }

    // act_fac_lut_tab
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("act_fac_lut_tab", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "act_fac_lut_tab") == 0)
    {
        __result = act_fac_lut_tab.GetInfo(&__path[__pos], __info);
    }

    // act_fac0
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("act_fac0", ParameterParserDataType::UInt32, (double)act_fac0, 0));
    }
    else if (PARAMETER_STRCMP(__name, "act_fac0") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::UInt32, (double)act_fac0, 0));
    }

    // act_fac1
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("act_fac1", ParameterParserDataType::UInt32, (double)act_fac1, 0));
    }
    else if (PARAMETER_STRCMP(__name, "act_fac1") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::UInt32, (double)act_fac1, 0));
    }

    // act_thd0
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("act_thd0", ParameterParserDataType::UInt32, (double)act_thd0, 0));
    }
    else if (PARAMETER_STRCMP(__name, "act_thd0") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::UInt32, (double)act_thd0, 0));
    }

    // act_thd1
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("act_thd1", ParameterParserDataType::UInt32, (double)act_thd1, 0));
    }
    else if (PARAMETER_STRCMP(__name, "act_thd1") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::UInt32, (double)act_thd1, 0));
    }

    // act_smth_thd0
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("act_smth_thd0", ParameterParserDataType::UInt32, (double)act_smth_thd0, 0));
    }
    else if (PARAMETER_STRCMP(__name, "act_smth_thd0") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::UInt32, (double)act_smth_thd0, 0));
    }

    // act_smth_thd1
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("act_smth_thd1", ParameterParserDataType::UInt32, (double)act_smth_thd1, 0));
    }
    else if (PARAMETER_STRCMP(__name, "act_smth_thd1") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::UInt32, (double)act_smth_thd1, 0));
    }

    // dark_thd
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("dark_thd", ParameterParserDataType::UInt32, (double)dark_thd, 0));
    }
    else if (PARAMETER_STRCMP(__name, "dark_thd") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::UInt32, (double)dark_thd, 0));
    }

    // dark_fac_lut_tab
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("dark_fac_lut_tab", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "dark_fac_lut_tab") == 0)
    {
        __result = dark_fac_lut_tab.GetInfo(&__path[__pos], __info);
    }

    // edge_detect_thd
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("edge_detect_thd", ParameterParserDataType::UInt32, (double)edge_detect_thd, 0));
    }
    else if (PARAMETER_STRCMP(__name, "edge_detect_thd") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::UInt32, (double)edge_detect_thd, 0));
    }

    // edge_count_low
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("edge_count_low", ParameterParserDataType::UInt32, (double)edge_count_low, 0));
    }
    else if (PARAMETER_STRCMP(__name, "edge_count_low") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::UInt32, (double)edge_count_low, 0));
    }

    // edge_detect_noise_scaler
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("edge_detect_noise_scaler", ParameterParserDataType::Float, (double)edge_detect_noise_scaler));
    }
    else if (PARAMETER_STRCMP(__name, "edge_detect_noise_scaler") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)edge_detect_noise_scaler));
    }

    // edge_smooth_strength
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("edge_smooth_strength", ParameterParserDataType::Float, (double)edge_smooth_strength));
    }
    else if (PARAMETER_STRCMP(__name, "edge_smooth_strength") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)edge_smooth_strength));
    }

    // edge_smooth_scaler_tab
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("edge_smooth_scaler_tab", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "edge_smooth_scaler_tab") == 0)
    {
        __result = edge_smooth_scaler_tab.GetInfo(&__path[__pos], __info);
    }

    return __result;
}

bool abf40_rgn_dataType::blkpix_lev_tabStruct::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "blkpix_lev") == 0)
    {
        if (__index < 2)
        {
            ParameterUtils::DoubleToString((double)blkpix_lev[__index], __value, __size, 0);
            __result = true;
        }
        else
        {
            SIZE_T __valuePos = 0;
            for (UINT32 i = 0; i < 2; i++)
            {
                if (i > 0)
                {
                    PARAMETER_STRCAT(__value, __size, ",");
                    __valuePos = PARAMETER_STRLEN(__value);
                }
                ParameterUtils::DoubleToString((double)blkpix_lev[i], &__value[__valuePos], (INT32)(__size - __valuePos), 0);
            }
            __result = true;
        }
    }

    return __result;
}

bool abf40_rgn_dataType::blkpix_lev_tabStruct::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "blkpix_lev") == 0)
    {
        if (__index < 2)
        {
            const double dblValue = ParameterUtils::StringToDouble(__value, __result);
            if (__result)
            {
                blkpix_lev[__index] = (float)dblValue; 
            }
        }
        else
        {
            SIZE_T __valueLen = PARAMETER_STRLEN(__value);
            UINT32 __pos = 0;

            __index = 0;
            while (__pos < __valueLen && __index < 2)
            {
                CHAR __valueStr[MAX_SIZE];

                __pos += ParameterUtils::GetSubString(&__value[__pos], ',', __valueStr, MAX_SIZE);
                __pos++;

                const double dblValue = ParameterUtils::StringToDouble(__valueStr, __result);
                if (__result)
                {
                    blkpix_lev[__index] = (float)dblValue; 
                }
                __index++;
            }
            __result = true;
        }
    }

    return __result;
}

bool abf40_rgn_dataType::blkpix_lev_tabStruct::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // blkpix_lev
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("blkpix_lev", ParameterParserDataType::UInt32, true, false, 2));
    }

    return __result;
}

bool abf40_rgn_dataType::noise_std_lut_tabStruct::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "noise_std_lut") == 0)
    {
        if (__index < 65)
        {
            ParameterUtils::DoubleToString((double)noise_std_lut[__index], __value, __size);
            __result = true;
        }
        else
        {
            SIZE_T __valuePos = 0;
            for (UINT32 i = 0; i < 65; i++)
            {
                if (i > 0)
                {
                    PARAMETER_STRCAT(__value, __size, ",");
                    __valuePos = PARAMETER_STRLEN(__value);
                }
                ParameterUtils::DoubleToString((double)noise_std_lut[i], &__value[__valuePos], (INT32)(__size - __valuePos));
            }
            __result = true;
        }
    }

    return __result;
}

bool abf40_rgn_dataType::noise_std_lut_tabStruct::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "noise_std_lut") == 0)
    {
        if (__index < 65)
        {
            const double dblValue = ParameterUtils::StringToDouble(__value, __result);
            if (__result)
            {
                noise_std_lut[__index] = (float)dblValue; 
            }
        }
        else
        {
            SIZE_T __valueLen = PARAMETER_STRLEN(__value);
            UINT32 __pos = 0;

            __index = 0;
            while (__pos < __valueLen && __index < 65)
            {
                CHAR __valueStr[MAX_SIZE];

                __pos += ParameterUtils::GetSubString(&__value[__pos], ',', __valueStr, MAX_SIZE);
                __pos++;

                const double dblValue = ParameterUtils::StringToDouble(__valueStr, __result);
                if (__result)
                {
                    noise_std_lut[__index] = (float)dblValue; 
                }
                __index++;
            }
            __result = true;
        }
    }

    return __result;
}

bool abf40_rgn_dataType::noise_std_lut_tabStruct::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // noise_std_lut
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("noise_std_lut", ParameterParserDataType::Float, true, false, 65));
    }

    return __result;
}

bool abf40_rgn_dataType::curve_offset_tabStruct::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "curve_offset") == 0)
    {
        if (__index < 4)
        {
            ParameterUtils::DoubleToString((double)curve_offset[__index], __value, __size, 0);
            __result = true;
        }
        else
        {
            SIZE_T __valuePos = 0;
            for (UINT32 i = 0; i < 4; i++)
            {
                if (i > 0)
                {
                    PARAMETER_STRCAT(__value, __size, ",");
                    __valuePos = PARAMETER_STRLEN(__value);
                }
                ParameterUtils::DoubleToString((double)curve_offset[i], &__value[__valuePos], (INT32)(__size - __valuePos), 0);
            }
            __result = true;
        }
    }

    return __result;
}

bool abf40_rgn_dataType::curve_offset_tabStruct::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "curve_offset") == 0)
    {
        if (__index < 4)
        {
            const double dblValue = ParameterUtils::StringToDouble(__value, __result);
            if (__result)
            {
                curve_offset[__index] = (float)dblValue; 
            }
        }
        else
        {
            SIZE_T __valueLen = PARAMETER_STRLEN(__value);
            UINT32 __pos = 0;

            __index = 0;
            while (__pos < __valueLen && __index < 4)
            {
                CHAR __valueStr[MAX_SIZE];

                __pos += ParameterUtils::GetSubString(&__value[__pos], ',', __valueStr, MAX_SIZE);
                __pos++;

                const double dblValue = ParameterUtils::StringToDouble(__valueStr, __result);
                if (__result)
                {
                    curve_offset[__index] = (float)dblValue; 
                }
                __index++;
            }
            __result = true;
        }
    }

    return __result;
}

bool abf40_rgn_dataType::curve_offset_tabStruct::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // curve_offset
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("curve_offset", ParameterParserDataType::UInt32, true, false, 4));
    }

    return __result;
}

bool abf40_rgn_dataType::dist_ker_tabStruct::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "dist_ker") == 0)
    {
        if (__index < 18)
        {
            ParameterUtils::DoubleToString((double)dist_ker[__index], __value, __size, 0);
            __result = true;
        }
        else
        {
            SIZE_T __valuePos = 0;
            for (UINT32 i = 0; i < 18; i++)
            {
                if (i > 0)
                {
                    PARAMETER_STRCAT(__value, __size, ",");
                    __valuePos = PARAMETER_STRLEN(__value);
                }
                ParameterUtils::DoubleToString((double)dist_ker[i], &__value[__valuePos], (INT32)(__size - __valuePos), 0);
            }
            __result = true;
        }
    }

    return __result;
}

bool abf40_rgn_dataType::dist_ker_tabStruct::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "dist_ker") == 0)
    {
        if (__index < 18)
        {
            const double dblValue = ParameterUtils::StringToDouble(__value, __result);
            if (__result)
            {
                dist_ker[__index] = (float)dblValue; 
            }
        }
        else
        {
            SIZE_T __valueLen = PARAMETER_STRLEN(__value);
            UINT32 __pos = 0;

            __index = 0;
            while (__pos < __valueLen && __index < 18)
            {
                CHAR __valueStr[MAX_SIZE];

                __pos += ParameterUtils::GetSubString(&__value[__pos], ',', __valueStr, MAX_SIZE);
                __pos++;

                const double dblValue = ParameterUtils::StringToDouble(__valueStr, __result);
                if (__result)
                {
                    dist_ker[__index] = (float)dblValue; 
                }
                __index++;
            }
            __result = true;
        }
    }

    return __result;
}

bool abf40_rgn_dataType::dist_ker_tabStruct::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // dist_ker
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("dist_ker", ParameterParserDataType::Int32, true, false, 18));
    }

    return __result;
}

bool abf40_rgn_dataType::edge_softness_tabStruct::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "edge_softness") == 0)
    {
        if (__index < 4)
        {
            ParameterUtils::DoubleToString((double)edge_softness[__index], __value, __size);
            __result = true;
        }
        else
        {
            SIZE_T __valuePos = 0;
            for (UINT32 i = 0; i < 4; i++)
            {
                if (i > 0)
                {
                    PARAMETER_STRCAT(__value, __size, ",");
                    __valuePos = PARAMETER_STRLEN(__value);
                }
                ParameterUtils::DoubleToString((double)edge_softness[i], &__value[__valuePos], (INT32)(__size - __valuePos));
            }
            __result = true;
        }
    }

    return __result;
}

bool abf40_rgn_dataType::edge_softness_tabStruct::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "edge_softness") == 0)
    {
        if (__index < 4)
        {
            const double dblValue = ParameterUtils::StringToDouble(__value, __result);
            if (__result)
            {
                edge_softness[__index] = (float)dblValue; 
            }
        }
        else
        {
            SIZE_T __valueLen = PARAMETER_STRLEN(__value);
            UINT32 __pos = 0;

            __index = 0;
            while (__pos < __valueLen && __index < 4)
            {
                CHAR __valueStr[MAX_SIZE];

                __pos += ParameterUtils::GetSubString(&__value[__pos], ',', __valueStr, MAX_SIZE);
                __pos++;

                const double dblValue = ParameterUtils::StringToDouble(__valueStr, __result);
                if (__result)
                {
                    edge_softness[__index] = (float)dblValue; 
                }
                __index++;
            }
            __result = true;
        }
    }

    return __result;
}

bool abf40_rgn_dataType::edge_softness_tabStruct::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // edge_softness
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("edge_softness", ParameterParserDataType::Float, true, false, 4));
    }

    return __result;
}

bool abf40_rgn_dataType::denoise_strength_tabStruct::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "denoise_strength") == 0)
    {
        if (__index < 4)
        {
            ParameterUtils::DoubleToString((double)denoise_strength[__index], __value, __size);
            __result = true;
        }
        else
        {
            SIZE_T __valuePos = 0;
            for (UINT32 i = 0; i < 4; i++)
            {
                if (i > 0)
                {
                    PARAMETER_STRCAT(__value, __size, ",");
                    __valuePos = PARAMETER_STRLEN(__value);
                }
                ParameterUtils::DoubleToString((double)denoise_strength[i], &__value[__valuePos], (INT32)(__size - __valuePos));
            }
            __result = true;
        }
    }

    return __result;
}

bool abf40_rgn_dataType::denoise_strength_tabStruct::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "denoise_strength") == 0)
    {
        if (__index < 4)
        {
            const double dblValue = ParameterUtils::StringToDouble(__value, __result);
            if (__result)
            {
                denoise_strength[__index] = (float)dblValue; 
            }
        }
        else
        {
            SIZE_T __valueLen = PARAMETER_STRLEN(__value);
            UINT32 __pos = 0;

            __index = 0;
            while (__pos < __valueLen && __index < 4)
            {
                CHAR __valueStr[MAX_SIZE];

                __pos += ParameterUtils::GetSubString(&__value[__pos], ',', __valueStr, MAX_SIZE);
                __pos++;

                const double dblValue = ParameterUtils::StringToDouble(__valueStr, __result);
                if (__result)
                {
                    denoise_strength[__index] = (float)dblValue; 
                }
                __index++;
            }
            __result = true;
        }
    }

    return __result;
}

bool abf40_rgn_dataType::denoise_strength_tabStruct::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // denoise_strength
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("denoise_strength", ParameterParserDataType::Float, true, false, 4));
    }

    return __result;
}

bool abf40_rgn_dataType::noise_prsv_anchor_tabStruct::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "noise_prsv_anchor") == 0)
    {
        if (__index < 5)
        {
            ParameterUtils::DoubleToString((double)noise_prsv_anchor[__index], __value, __size);
            __result = true;
        }
        else
        {
            SIZE_T __valuePos = 0;
            for (UINT32 i = 0; i < 5; i++)
            {
                if (i > 0)
                {
                    PARAMETER_STRCAT(__value, __size, ",");
                    __valuePos = PARAMETER_STRLEN(__value);
                }
                ParameterUtils::DoubleToString((double)noise_prsv_anchor[i], &__value[__valuePos], (INT32)(__size - __valuePos));
            }
            __result = true;
        }
    }

    return __result;
}

bool abf40_rgn_dataType::noise_prsv_anchor_tabStruct::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "noise_prsv_anchor") == 0)
    {
        if (__index < 5)
        {
            const double dblValue = ParameterUtils::StringToDouble(__value, __result);
            if (__result)
            {
                noise_prsv_anchor[__index] = (float)dblValue; 
            }
        }
        else
        {
            SIZE_T __valueLen = PARAMETER_STRLEN(__value);
            UINT32 __pos = 0;

            __index = 0;
            while (__pos < __valueLen && __index < 5)
            {
                CHAR __valueStr[MAX_SIZE];

                __pos += ParameterUtils::GetSubString(&__value[__pos], ',', __valueStr, MAX_SIZE);
                __pos++;

                const double dblValue = ParameterUtils::StringToDouble(__valueStr, __result);
                if (__result)
                {
                    noise_prsv_anchor[__index] = (float)dblValue; 
                }
                __index++;
            }
            __result = true;
        }
    }

    return __result;
}

bool abf40_rgn_dataType::noise_prsv_anchor_tabStruct::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // noise_prsv_anchor
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("noise_prsv_anchor", ParameterParserDataType::Float, true, false, 5));
    }

    return __result;
}

bool abf40_rgn_dataType::noise_prsv_base_tabStruct::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "noise_prsv_base") == 0)
    {
        if (__index < 10)
        {
            ParameterUtils::DoubleToString((double)noise_prsv_base[__index], __value, __size);
            __result = true;
        }
        else
        {
            SIZE_T __valuePos = 0;
            for (UINT32 i = 0; i < 10; i++)
            {
                if (i > 0)
                {
                    PARAMETER_STRCAT(__value, __size, ",");
                    __valuePos = PARAMETER_STRLEN(__value);
                }
                ParameterUtils::DoubleToString((double)noise_prsv_base[i], &__value[__valuePos], (INT32)(__size - __valuePos));
            }
            __result = true;
        }
    }

    return __result;
}

bool abf40_rgn_dataType::noise_prsv_base_tabStruct::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "noise_prsv_base") == 0)
    {
        if (__index < 10)
        {
            const double dblValue = ParameterUtils::StringToDouble(__value, __result);
            if (__result)
            {
                noise_prsv_base[__index] = (float)dblValue; 
            }
        }
        else
        {
            SIZE_T __valueLen = PARAMETER_STRLEN(__value);
            UINT32 __pos = 0;

            __index = 0;
            while (__pos < __valueLen && __index < 10)
            {
                CHAR __valueStr[MAX_SIZE];

                __pos += ParameterUtils::GetSubString(&__value[__pos], ',', __valueStr, MAX_SIZE);
                __pos++;

                const double dblValue = ParameterUtils::StringToDouble(__valueStr, __result);
                if (__result)
                {
                    noise_prsv_base[__index] = (float)dblValue; 
                }
                __index++;
            }
            __result = true;
        }
    }

    return __result;
}

bool abf40_rgn_dataType::noise_prsv_base_tabStruct::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // noise_prsv_base
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("noise_prsv_base", ParameterParserDataType::Float, true, false, 10));
    }

    return __result;
}

bool abf40_rgn_dataType::radial_edge_softness_adj_tabStruct::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "radial_edge_softness_adj") == 0)
    {
        if (__index < 5)
        {
            ParameterUtils::DoubleToString((double)radial_edge_softness_adj[__index], __value, __size);
            __result = true;
        }
        else
        {
            SIZE_T __valuePos = 0;
            for (UINT32 i = 0; i < 5; i++)
            {
                if (i > 0)
                {
                    PARAMETER_STRCAT(__value, __size, ",");
                    __valuePos = PARAMETER_STRLEN(__value);
                }
                ParameterUtils::DoubleToString((double)radial_edge_softness_adj[i], &__value[__valuePos], (INT32)(__size - __valuePos));
            }
            __result = true;
        }
    }

    return __result;
}

bool abf40_rgn_dataType::radial_edge_softness_adj_tabStruct::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "radial_edge_softness_adj") == 0)
    {
        if (__index < 5)
        {
            const double dblValue = ParameterUtils::StringToDouble(__value, __result);
            if (__result)
            {
                radial_edge_softness_adj[__index] = (float)dblValue; 
            }
        }
        else
        {
            SIZE_T __valueLen = PARAMETER_STRLEN(__value);
            UINT32 __pos = 0;

            __index = 0;
            while (__pos < __valueLen && __index < 5)
            {
                CHAR __valueStr[MAX_SIZE];

                __pos += ParameterUtils::GetSubString(&__value[__pos], ',', __valueStr, MAX_SIZE);
                __pos++;

                const double dblValue = ParameterUtils::StringToDouble(__valueStr, __result);
                if (__result)
                {
                    radial_edge_softness_adj[__index] = (float)dblValue; 
                }
                __index++;
            }
            __result = true;
        }
    }

    return __result;
}

bool abf40_rgn_dataType::radial_edge_softness_adj_tabStruct::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // radial_edge_softness_adj
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("radial_edge_softness_adj", ParameterParserDataType::Float, true, false, 5));
    }

    return __result;
}

bool abf40_rgn_dataType::radial_noise_prsv_adj_tabStruct::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "radial_noise_prsv_adj") == 0)
    {
        if (__index < 5)
        {
            ParameterUtils::DoubleToString((double)radial_noise_prsv_adj[__index], __value, __size);
            __result = true;
        }
        else
        {
            SIZE_T __valuePos = 0;
            for (UINT32 i = 0; i < 5; i++)
            {
                if (i > 0)
                {
                    PARAMETER_STRCAT(__value, __size, ",");
                    __valuePos = PARAMETER_STRLEN(__value);
                }
                ParameterUtils::DoubleToString((double)radial_noise_prsv_adj[i], &__value[__valuePos], (INT32)(__size - __valuePos));
            }
            __result = true;
        }
    }

    return __result;
}

bool abf40_rgn_dataType::radial_noise_prsv_adj_tabStruct::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "radial_noise_prsv_adj") == 0)
    {
        if (__index < 5)
        {
            const double dblValue = ParameterUtils::StringToDouble(__value, __result);
            if (__result)
            {
                radial_noise_prsv_adj[__index] = (float)dblValue; 
            }
        }
        else
        {
            SIZE_T __valueLen = PARAMETER_STRLEN(__value);
            UINT32 __pos = 0;

            __index = 0;
            while (__pos < __valueLen && __index < 5)
            {
                CHAR __valueStr[MAX_SIZE];

                __pos += ParameterUtils::GetSubString(&__value[__pos], ',', __valueStr, MAX_SIZE);
                __pos++;

                const double dblValue = ParameterUtils::StringToDouble(__valueStr, __result);
                if (__result)
                {
                    radial_noise_prsv_adj[__index] = (float)dblValue; 
                }
                __index++;
            }
            __result = true;
        }
    }

    return __result;
}

bool abf40_rgn_dataType::radial_noise_prsv_adj_tabStruct::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // radial_noise_prsv_adj
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("radial_noise_prsv_adj", ParameterParserDataType::Float, true, false, 5));
    }

    return __result;
}

bool abf40_rgn_dataType::act_fac_lut_tabStruct::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "act_fac_lut") == 0)
    {
        if (__index < 32)
        {
            ParameterUtils::DoubleToString((double)act_fac_lut[__index], __value, __size, 0);
            __result = true;
        }
        else
        {
            SIZE_T __valuePos = 0;
            for (UINT32 i = 0; i < 32; i++)
            {
                if (i > 0)
                {
                    PARAMETER_STRCAT(__value, __size, ",");
                    __valuePos = PARAMETER_STRLEN(__value);
                }
                ParameterUtils::DoubleToString((double)act_fac_lut[i], &__value[__valuePos], (INT32)(__size - __valuePos), 0);
            }
            __result = true;
        }
    }

    return __result;
}

bool abf40_rgn_dataType::act_fac_lut_tabStruct::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "act_fac_lut") == 0)
    {
        if (__index < 32)
        {
            const double dblValue = ParameterUtils::StringToDouble(__value, __result);
            if (__result)
            {
                act_fac_lut[__index] = (float)dblValue; 
            }
        }
        else
        {
            SIZE_T __valueLen = PARAMETER_STRLEN(__value);
            UINT32 __pos = 0;

            __index = 0;
            while (__pos < __valueLen && __index < 32)
            {
                CHAR __valueStr[MAX_SIZE];

                __pos += ParameterUtils::GetSubString(&__value[__pos], ',', __valueStr, MAX_SIZE);
                __pos++;

                const double dblValue = ParameterUtils::StringToDouble(__valueStr, __result);
                if (__result)
                {
                    act_fac_lut[__index] = (float)dblValue; 
                }
                __index++;
            }
            __result = true;
        }
    }

    return __result;
}

bool abf40_rgn_dataType::act_fac_lut_tabStruct::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // act_fac_lut
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("act_fac_lut", ParameterParserDataType::UInt32, true, false, 32));
    }

    return __result;
}

bool abf40_rgn_dataType::dark_fac_lut_tabStruct::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "dark_fac_lut") == 0)
    {
        if (__index < 42)
        {
            ParameterUtils::DoubleToString((double)dark_fac_lut[__index], __value, __size, 0);
            __result = true;
        }
        else
        {
            SIZE_T __valuePos = 0;
            for (UINT32 i = 0; i < 42; i++)
            {
                if (i > 0)
                {
                    PARAMETER_STRCAT(__value, __size, ",");
                    __valuePos = PARAMETER_STRLEN(__value);
                }
                ParameterUtils::DoubleToString((double)dark_fac_lut[i], &__value[__valuePos], (INT32)(__size - __valuePos), 0);
            }
            __result = true;
        }
    }

    return __result;
}

bool abf40_rgn_dataType::dark_fac_lut_tabStruct::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "dark_fac_lut") == 0)
    {
        if (__index < 42)
        {
            const double dblValue = ParameterUtils::StringToDouble(__value, __result);
            if (__result)
            {
                dark_fac_lut[__index] = (float)dblValue; 
            }
        }
        else
        {
            SIZE_T __valueLen = PARAMETER_STRLEN(__value);
            UINT32 __pos = 0;

            __index = 0;
            while (__pos < __valueLen && __index < 42)
            {
                CHAR __valueStr[MAX_SIZE];

                __pos += ParameterUtils::GetSubString(&__value[__pos], ',', __valueStr, MAX_SIZE);
                __pos++;

                const double dblValue = ParameterUtils::StringToDouble(__valueStr, __result);
                if (__result)
                {
                    dark_fac_lut[__index] = (float)dblValue; 
                }
                __index++;
            }
            __result = true;
        }
    }

    return __result;
}

bool abf40_rgn_dataType::dark_fac_lut_tabStruct::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // dark_fac_lut
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("dark_fac_lut", ParameterParserDataType::UInt32, true, false, 42));
    }

    return __result;
}

bool abf40_rgn_dataType::edge_smooth_scaler_tabStruct::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "edge_smooth_scaler") == 0)
    {
        if (__index < 4)
        {
            ParameterUtils::DoubleToString((double)edge_smooth_scaler[__index], __value, __size);
            __result = true;
        }
        else
        {
            SIZE_T __valuePos = 0;
            for (UINT32 i = 0; i < 4; i++)
            {
                if (i > 0)
                {
                    PARAMETER_STRCAT(__value, __size, ",");
                    __valuePos = PARAMETER_STRLEN(__value);
                }
                ParameterUtils::DoubleToString((double)edge_smooth_scaler[i], &__value[__valuePos], (INT32)(__size - __valuePos));
            }
            __result = true;
        }
    }

    return __result;
}

bool abf40_rgn_dataType::edge_smooth_scaler_tabStruct::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "edge_smooth_scaler") == 0)
    {
        if (__index < 4)
        {
            const double dblValue = ParameterUtils::StringToDouble(__value, __result);
            if (__result)
            {
                edge_smooth_scaler[__index] = (float)dblValue; 
            }
        }
        else
        {
            SIZE_T __valueLen = PARAMETER_STRLEN(__value);
            UINT32 __pos = 0;

            __index = 0;
            while (__pos < __valueLen && __index < 4)
            {
                CHAR __valueStr[MAX_SIZE];

                __pos += ParameterUtils::GetSubString(&__value[__pos], ',', __valueStr, MAX_SIZE);
                __pos++;

                const double dblValue = ParameterUtils::StringToDouble(__valueStr, __result);
                if (__result)
                {
                    edge_smooth_scaler[__index] = (float)dblValue; 
                }
                __index++;
            }
            __result = true;
        }
    }

    return __result;
}

bool abf40_rgn_dataType::edge_smooth_scaler_tabStruct::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // edge_smooth_scaler
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("edge_smooth_scaler", ParameterParserDataType::Float, true, false, 4));
    }

    return __result;
}

bool mod_abf40_aec_dataType::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "aec_trigger") == 0)
    {
        __result = aec_trigger.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "abf40_rgn_data") == 0)
    {
        __result = abf40_rgn_data.GetValue(&__path[__pos], __value, __size);
    }

    return __result;
}

bool mod_abf40_aec_dataType::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "aec_trigger") == 0)
    {
        __result = aec_trigger.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "abf40_rgn_data") == 0)
    {
        __result = abf40_rgn_data.SetValue(&__path[__pos], __value);
    }

    return __result;
}

bool mod_abf40_aec_dataType::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // aec_trigger
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("aec_trigger", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "aec_trigger") == 0)
    {
        __result = aec_trigger.GetInfo(&__path[__pos], __info);
    }

    // abf40_rgn_data
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("abf40_rgn_data", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "abf40_rgn_data") == 0)
    {
        __result = abf40_rgn_data.GetInfo(&__path[__pos], __info);
    }

    return __result;
}

bool mod_abf40_hdr_aec_dataType::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "hdr_aec_trigger") == 0)
    {
        __result = hdr_aec_trigger.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "hdr_aec_data") == 0)
    {
        __result = hdr_aec_data.GetValue(&__path[__pos], __value, __size);
    }

    return __result;
}

bool mod_abf40_hdr_aec_dataType::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "hdr_aec_trigger") == 0)
    {
        __result = hdr_aec_trigger.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "hdr_aec_data") == 0)
    {
        __result = hdr_aec_data.SetValue(&__path[__pos], __value);
    }

    return __result;
}

bool mod_abf40_hdr_aec_dataType::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // hdr_aec_trigger
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("hdr_aec_trigger", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "hdr_aec_trigger") == 0)
    {
        __result = hdr_aec_trigger.GetInfo(&__path[__pos], __info);
    }

    // hdr_aec_data
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("hdr_aec_data", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "hdr_aec_data") == 0)
    {
        __result = hdr_aec_data.GetInfo(&__path[__pos], __info);
    }

    return __result;
}

bool mod_abf40_hdr_aec_dataType::hdr_aec_dataStruct::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "mod_abf40_aec_data") == 0)
    {
        if (__index < mod_abf40_aec_dataCount)
        {
            __result = mod_abf40_aec_data[__index].GetValue(&__path[__pos], __value, __size);
        }
    }

    return __result;
}

bool mod_abf40_hdr_aec_dataType::hdr_aec_dataStruct::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "mod_abf40_aec_data") == 0)
    {
        if (__index < mod_abf40_aec_dataCount)
        {
            __result = mod_abf40_aec_data[__index].SetValue(&__path[__pos], __value);
        }
    }

    return __result;
}

bool mod_abf40_hdr_aec_dataType::hdr_aec_dataStruct::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // mod_abf40_aec_data
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("mod_abf40_aec_data", ParameterParserDataType::Structure, true, false, mod_abf40_aec_dataCount));
    }
    else if (PARAMETER_STRCMP(__name, "mod_abf40_aec_data") == 0)
    {
        if (__index < mod_abf40_aec_dataCount)
        {
            __result = mod_abf40_aec_data[__index].GetInfo(&__path[__pos], __info);
        }
        else
        {
            __result = true;
            __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Structure, true, false, mod_abf40_aec_dataCount));
        }
    }

    return __result;
}

bool mod_abf40_drc_gain_dataType::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "drc_gain_trigger") == 0)
    {
        __result = drc_gain_trigger.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "drc_gain_data") == 0)
    {
        __result = drc_gain_data.GetValue(&__path[__pos], __value, __size);
    }

    return __result;
}

bool mod_abf40_drc_gain_dataType::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "drc_gain_trigger") == 0)
    {
        __result = drc_gain_trigger.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "drc_gain_data") == 0)
    {
        __result = drc_gain_data.SetValue(&__path[__pos], __value);
    }

    return __result;
}

bool mod_abf40_drc_gain_dataType::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // drc_gain_trigger
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("drc_gain_trigger", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "drc_gain_trigger") == 0)
    {
        __result = drc_gain_trigger.GetInfo(&__path[__pos], __info);
    }

    // drc_gain_data
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("drc_gain_data", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "drc_gain_data") == 0)
    {
        __result = drc_gain_data.GetInfo(&__path[__pos], __info);
    }

    return __result;
}

bool mod_abf40_drc_gain_dataType::drc_gain_dataStruct::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "mod_abf40_hdr_aec_data") == 0)
    {
        if (__index < mod_abf40_hdr_aec_dataCount)
        {
            __result = mod_abf40_hdr_aec_data[__index].GetValue(&__path[__pos], __value, __size);
        }
    }

    return __result;
}

bool mod_abf40_drc_gain_dataType::drc_gain_dataStruct::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "mod_abf40_hdr_aec_data") == 0)
    {
        if (__index < mod_abf40_hdr_aec_dataCount)
        {
            __result = mod_abf40_hdr_aec_data[__index].SetValue(&__path[__pos], __value);
        }
    }

    return __result;
}

bool mod_abf40_drc_gain_dataType::drc_gain_dataStruct::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // mod_abf40_hdr_aec_data
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("mod_abf40_hdr_aec_data", ParameterParserDataType::Structure, true, false, mod_abf40_hdr_aec_dataCount));
    }
    else if (PARAMETER_STRCMP(__name, "mod_abf40_hdr_aec_data") == 0)
    {
        if (__index < mod_abf40_hdr_aec_dataCount)
        {
            __result = mod_abf40_hdr_aec_data[__index].GetInfo(&__path[__pos], __info);
        }
        else
        {
            __result = true;
            __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Structure, true, false, mod_abf40_hdr_aec_dataCount));
        }
    }

    return __result;
}

bool chromatix_abf40_coreType::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "mod_abf40_drc_gain_data") == 0)
    {
        if (__index < mod_abf40_drc_gain_dataCount)
        {
            __result = mod_abf40_drc_gain_data[__index].GetValue(&__path[__pos], __value, __size);
        }
    }

    return __result;
}

bool chromatix_abf40_coreType::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "mod_abf40_drc_gain_data") == 0)
    {
        if (__index < mod_abf40_drc_gain_dataCount)
        {
            __result = mod_abf40_drc_gain_data[__index].SetValue(&__path[__pos], __value);
        }
    }

    return __result;
}

bool chromatix_abf40_coreType::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // mod_abf40_drc_gain_data
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("mod_abf40_drc_gain_data", ParameterParserDataType::Structure, true, false, mod_abf40_drc_gain_dataCount));
    }
    else if (PARAMETER_STRCMP(__name, "mod_abf40_drc_gain_data") == 0)
    {
        if (__index < mod_abf40_drc_gain_dataCount)
        {
            __result = mod_abf40_drc_gain_data[__index].GetInfo(&__path[__pos], __info);
        }
        else
        {
            __result = true;
            __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Structure, true, false, mod_abf40_drc_gain_dataCount));
        }
    }

    return __result;
}

bool chromatix_abf40_reserveType::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "cross_plane_en") == 0)
    {
        ParameterUtils::DoubleToString((double)cross_plane_en, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "dark_desat_en") == 0)
    {
        ParameterUtils::DoubleToString((double)dark_desat_en, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "dark_smooth_en") == 0)
    {
        ParameterUtils::DoubleToString((double)dark_smooth_en, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "act_adj_en") == 0)
    {
        ParameterUtils::DoubleToString((double)act_adj_en, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "blk_opt") == 0)
    {
        ParameterUtils::DoubleToString((double)blk_opt, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "radial_anchor_tab") == 0)
    {
        __result = radial_anchor_tab.GetValue(&__path[__pos], __value, __size);
    }

    return __result;
}

bool chromatix_abf40_reserveType::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "cross_plane_en") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            cross_plane_en = (globalelements::enable_flag_type)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "dark_desat_en") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            dark_desat_en = (globalelements::enable_flag_type)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "dark_smooth_en") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            dark_smooth_en = (globalelements::enable_flag_type)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "act_adj_en") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            act_adj_en = (globalelements::enable_flag_type)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "blk_opt") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            blk_opt = (INT32)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "radial_anchor_tab") == 0)
    {
        __result = radial_anchor_tab.SetValue(&__path[__pos], __value);
    }

    return __result;
}

bool chromatix_abf40_reserveType::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // cross_plane_en
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("cross_plane_en", ParameterParserDataType::Int32, (double)cross_plane_en, 0));
    }
    else if (PARAMETER_STRCMP(__name, "cross_plane_en") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Int32, (double)cross_plane_en, 0));
    }

    // dark_desat_en
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("dark_desat_en", ParameterParserDataType::Int32, (double)dark_desat_en, 0));
    }
    else if (PARAMETER_STRCMP(__name, "dark_desat_en") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Int32, (double)dark_desat_en, 0));
    }

    // dark_smooth_en
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("dark_smooth_en", ParameterParserDataType::Int32, (double)dark_smooth_en, 0));
    }
    else if (PARAMETER_STRCMP(__name, "dark_smooth_en") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Int32, (double)dark_smooth_en, 0));
    }

    // act_adj_en
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("act_adj_en", ParameterParserDataType::Int32, (double)act_adj_en, 0));
    }
    else if (PARAMETER_STRCMP(__name, "act_adj_en") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Int32, (double)act_adj_en, 0));
    }

    // blk_opt
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("blk_opt", ParameterParserDataType::Int32, (double)blk_opt, 0));
    }
    else if (PARAMETER_STRCMP(__name, "blk_opt") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Int32, (double)blk_opt, 0));
    }

    // radial_anchor_tab
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("radial_anchor_tab", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "radial_anchor_tab") == 0)
    {
        __result = radial_anchor_tab.GetInfo(&__path[__pos], __info);
    }

    return __result;
}

bool chromatix_abf40_reserveType::radial_anchor_tabStruct::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "radial_anchor") == 0)
    {
        if (__index < 5)
        {
            ParameterUtils::DoubleToString((double)radial_anchor[__index], __value, __size);
            __result = true;
        }
        else
        {
            SIZE_T __valuePos = 0;
            for (UINT32 i = 0; i < 5; i++)
            {
                if (i > 0)
                {
                    PARAMETER_STRCAT(__value, __size, ",");
                    __valuePos = PARAMETER_STRLEN(__value);
                }
                ParameterUtils::DoubleToString((double)radial_anchor[i], &__value[__valuePos], (INT32)(__size - __valuePos));
            }
            __result = true;
        }
    }

    return __result;
}

bool chromatix_abf40_reserveType::radial_anchor_tabStruct::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "radial_anchor") == 0)
    {
        if (__index < 5)
        {
            const double dblValue = ParameterUtils::StringToDouble(__value, __result);
            if (__result)
            {
                radial_anchor[__index] = (float)dblValue; 
            }
        }
        else
        {
            SIZE_T __valueLen = PARAMETER_STRLEN(__value);
            UINT32 __pos = 0;

            __index = 0;
            while (__pos < __valueLen && __index < 5)
            {
                CHAR __valueStr[MAX_SIZE];

                __pos += ParameterUtils::GetSubString(&__value[__pos], ',', __valueStr, MAX_SIZE);
                __pos++;

                const double dblValue = ParameterUtils::StringToDouble(__valueStr, __result);
                if (__result)
                {
                    radial_anchor[__index] = (float)dblValue; 
                }
                __index++;
            }
            __result = true;
        }
    }

    return __result;
}

bool chromatix_abf40_reserveType::radial_anchor_tabStruct::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // radial_anchor
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("radial_anchor", ParameterParserDataType::Float, true, false, 5));
    }

    return __result;
}

bool chromatix_abf40Type::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "enable_section") == 0)
    {
        __result = enable_section.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "module_version") == 0)
    {
        __result = module_version.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "parameter_version") == 0)
    {
        __result = parameter_version.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "control_variables") == 0)
    {
        __result = control_variables.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "control_method") == 0)
    {
        __result = control_method.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "dynamic_enable_triggers") == 0)
    {
        __result = dynamic_enable_triggers.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "chromatix_abf40_reserve") == 0)
    {
        __result = chromatix_abf40_reserve.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "chromatix_abf40_core") == 0)
    {
        __result = chromatix_abf40_core.GetValue(&__path[__pos], __value, __size);
    }

    return __result;
}

bool chromatix_abf40Type::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "enable_section") == 0)
    {
        __result = enable_section.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "module_version") == 0)
    {
        __result = module_version.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "parameter_version") == 0)
    {
        __result = parameter_version.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "control_variables") == 0)
    {
        __result = control_variables.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "control_method") == 0)
    {
        __result = control_method.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "dynamic_enable_triggers") == 0)
    {
        __result = dynamic_enable_triggers.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "chromatix_abf40_reserve") == 0)
    {
        __result = chromatix_abf40_reserve.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "chromatix_abf40_core") == 0)
    {
        __result = chromatix_abf40_core.SetValue(&__path[__pos], __value);
    }

    return __result;
}

bool chromatix_abf40Type::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // enable_section
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("enable_section", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "enable_section") == 0)
    {
        __result = enable_section.GetInfo(&__path[__pos], __info);
    }

    // module_version
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("module_version", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "module_version") == 0)
    {
        __result = module_version.GetInfo(&__path[__pos], __info);
    }

    // parameter_version
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("parameter_version", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "parameter_version") == 0)
    {
        __result = parameter_version.GetInfo(&__path[__pos], __info);
    }

    // control_variables
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("control_variables", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "control_variables") == 0)
    {
        __result = control_variables.GetInfo(&__path[__pos], __info);
    }

    // control_method
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("control_method", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "control_method") == 0)
    {
        __result = control_method.GetInfo(&__path[__pos], __info);
    }

    // dynamic_enable_triggers
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("dynamic_enable_triggers", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "dynamic_enable_triggers") == 0)
    {
        __result = dynamic_enable_triggers.GetInfo(&__path[__pos], __info);
    }

    // chromatix_abf40_reserve
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("chromatix_abf40_reserve", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "chromatix_abf40_reserve") == 0)
    {
        __result = chromatix_abf40_reserve.GetInfo(&__path[__pos], __info);
    }

    // chromatix_abf40_core
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("chromatix_abf40_core", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "chromatix_abf40_core") == 0)
    {
        __result = chromatix_abf40_core.GetInfo(&__path[__pos], __info);
    }

    return __result;
}

bool chromatix_abf40Type::enable_sectionStruct::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "bilateral_en") == 0)
    {
        ParameterUtils::DoubleToString((double)bilateral_en, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "minmax_en") == 0)
    {
        ParameterUtils::DoubleToString((double)minmax_en, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "dirsmth_en") == 0)
    {
        ParameterUtils::DoubleToString((double)dirsmth_en, __value, __size, 0);
        __result = true;
    }

    return __result;
}

bool chromatix_abf40Type::enable_sectionStruct::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "bilateral_en") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            bilateral_en = (globalelements::enable_flag_type)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "minmax_en") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            minmax_en = (globalelements::enable_flag_type)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "dirsmth_en") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            dirsmth_en = (globalelements::enable_flag_type)dblValue; 
        }
    }

    return __result;
}

bool chromatix_abf40Type::enable_sectionStruct::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // bilateral_en
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("bilateral_en", ParameterParserDataType::Int32, (double)bilateral_en, 0));
    }
    else if (PARAMETER_STRCMP(__name, "bilateral_en") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Int32, (double)bilateral_en, 0));
    }

    // minmax_en
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("minmax_en", ParameterParserDataType::Int32, (double)minmax_en, 0));
    }
    else if (PARAMETER_STRCMP(__name, "minmax_en") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Int32, (double)minmax_en, 0));
    }

    // dirsmth_en
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("dirsmth_en", ParameterParserDataType::Int32, (double)dirsmth_en, 0));
    }
    else if (PARAMETER_STRCMP(__name, "dirsmth_en") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Int32, (double)dirsmth_en, 0));
    }

    return __result;
}

bool chromatix_abf40Type::module_versionStruct::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "major_revision") == 0)
    {
        ParameterUtils::DoubleToString((double)major_revision, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "minor_revision") == 0)
    {
        ParameterUtils::DoubleToString((double)minor_revision, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "incr_revision") == 0)
    {
        ParameterUtils::DoubleToString((double)incr_revision, __value, __size, 0);
        __result = true;
    }

    return __result;
}

bool chromatix_abf40Type::module_versionStruct::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "major_revision") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            major_revision = (INT32)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "minor_revision") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            minor_revision = (INT32)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "incr_revision") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            incr_revision = (INT32)dblValue; 
        }
    }

    return __result;
}

bool chromatix_abf40Type::module_versionStruct::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // major_revision
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("major_revision", ParameterParserDataType::Int32, (double)major_revision, 0));
    }
    else if (PARAMETER_STRCMP(__name, "major_revision") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Int32, (double)major_revision, 0));
    }

    // minor_revision
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("minor_revision", ParameterParserDataType::Int32, (double)minor_revision, 0));
    }
    else if (PARAMETER_STRCMP(__name, "minor_revision") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Int32, (double)minor_revision, 0));
    }

    // incr_revision
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("incr_revision", ParameterParserDataType::Int32, (double)incr_revision, 0));
    }
    else if (PARAMETER_STRCMP(__name, "incr_revision") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Int32, (double)incr_revision, 0));
    }

    return __result;
}

bool chromatix_abf40Type::parameter_versionStruct::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "revision") == 0)
    {
        PARAMETER_STRCPY(__value, __size, revision);
        __result = true;
    }

    return __result;
}

bool chromatix_abf40Type::parameter_versionStruct::SetValue(
    const CHAR* const __path,
    const CHAR* const /*__value*/)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "revision") == 0)
    {
        // Cannot modify strings, risk of memory leaks or corruption
    }

    return __result;
}

bool chromatix_abf40Type::parameter_versionStruct::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // revision
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("revision", ParameterParserDataType::String, revision));
    }
    else if (PARAMETER_STRCMP(__name, "revision") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::String, revision));
    }

    return __result;
}

bool chromatix_abf40Type::control_variablesStruct::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "control_var_type") == 0)
    {
        if (__index < 3)
        {
            ParameterUtils::DoubleToString((double)(UINT32)control_var_type[__index], __value, __size, 0);
            __result = true;
        }
        else
        {
            SIZE_T __valuePos = 0;
            for (UINT32 i = 0; i < 3; i++)
            {
                if (i > 0)
                {
                    PARAMETER_STRCAT(__value, __size, ",");
                    __valuePos = PARAMETER_STRLEN(__value);
                }
                ParameterUtils::DoubleToString((double)(UINT32)control_var_type[i], &__value[__valuePos], (INT32)(__size - __valuePos), 0);
            }
            __result = true;
        }
    }

    return __result;
}

bool chromatix_abf40Type::control_variablesStruct::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "control_var_type") == 0)
    {
        if (__index < 3)
        {
            const double dblValue = ParameterUtils::StringToDouble(__value, __result);
            if (__result)
            {
                control_var_type[__index] = (control_var_typeEnum)(UINT32)dblValue; 
            }
        }
        else
        {
            SIZE_T __valueLen = PARAMETER_STRLEN(__value);
            UINT32 __pos = 0;

            __index = 0;
            while (__pos < __valueLen && __index < 3)
            {
                CHAR __valueStr[MAX_SIZE];

                __pos += ParameterUtils::GetSubString(&__value[__pos], ',', __valueStr, MAX_SIZE);
                __pos++;

                const double dblValue = ParameterUtils::StringToDouble(__valueStr, __result);
                if (__result)
                {
                    control_var_type[__index] = (control_var_typeEnum)(UINT32)dblValue; 
                }
                __index++;
            }
            __result = true;
        }
    }

    return __result;
}

bool chromatix_abf40Type::control_variablesStruct::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // control_var_type
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("control_var_type", ParameterParserDataType::Enum, true, false, 3));
    }

    return __result;
}

bool chromatix_abf40Type::control_methodStruct::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "aec_exp_control") == 0)
    {
        ParameterUtils::DoubleToString((double)(UINT32)aec_exp_control, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "aec_hdr_control") == 0)
    {
        ParameterUtils::DoubleToString((double)(UINT32)aec_hdr_control, __value, __size, 0);
        __result = true;
    }

    return __result;
}

bool chromatix_abf40Type::control_methodStruct::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "aec_exp_control") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            aec_exp_control = (ispglobalelements::tuning_control_aec_type)(UINT32)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "aec_hdr_control") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            aec_hdr_control = (ispglobalelements::tuning_control_aec_hdr_type)(UINT32)dblValue; 
        }
    }

    return __result;
}

bool chromatix_abf40Type::control_methodStruct::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // aec_exp_control
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("aec_exp_control", ParameterParserDataType::Enum, (double)(UINT32)aec_exp_control, 0));
    }
    else if (PARAMETER_STRCMP(__name, "aec_exp_control") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Enum, (double)(UINT32)aec_exp_control, 0));
    }

    // aec_hdr_control
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("aec_hdr_control", ParameterParserDataType::Enum, (double)(UINT32)aec_hdr_control, 0));
    }
    else if (PARAMETER_STRCMP(__name, "aec_hdr_control") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Enum, (double)(UINT32)aec_hdr_control, 0));
    }

    return __result;
}

bool chromatix_abf40Type::dynamic_enable_triggersStruct::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "bilateral_en") == 0)
    {
        __result = bilateral_en.GetValue(&__path[__pos], __value, __size);
    }

    return __result;
}

bool chromatix_abf40Type::dynamic_enable_triggersStruct::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "bilateral_en") == 0)
    {
        __result = bilateral_en.SetValue(&__path[__pos], __value);
    }

    return __result;
}

bool chromatix_abf40Type::dynamic_enable_triggersStruct::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // bilateral_en
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("bilateral_en", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "bilateral_en") == 0)
    {
        __result = bilateral_en.GetInfo(&__path[__pos], __info);
    }

    return __result;
}

bool chromatix_abf40Type::dynamic_enable_triggersStruct::bilateral_enStruct::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "enable") == 0)
    {
        ParameterUtils::DoubleToString((double)enable, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "hyst_control_var") == 0)
    {
        ParameterUtils::DoubleToString((double)(UINT32)hyst_control_var, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "hyst_mode") == 0)
    {
        ParameterUtils::DoubleToString((double)(UINT32)hyst_mode, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "hyst_trigger") == 0)
    {
        __result = hyst_trigger.GetValue(&__path[__pos], __value, __size);
    }

    return __result;
}

bool chromatix_abf40Type::dynamic_enable_triggersStruct::bilateral_enStruct::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "enable") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            enable = (globalelements::enable_flag_type)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "hyst_control_var") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            hyst_control_var = (ispglobalelements::control_var_type)(UINT32)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "hyst_mode") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            hyst_mode = (ispglobalelements::hyst_direction)(UINT32)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "hyst_trigger") == 0)
    {
        __result = hyst_trigger.SetValue(&__path[__pos], __value);
    }

    return __result;
}

bool chromatix_abf40Type::dynamic_enable_triggersStruct::bilateral_enStruct::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // enable
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("enable", ParameterParserDataType::Int32, (double)enable, 0));
    }
    else if (PARAMETER_STRCMP(__name, "enable") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Int32, (double)enable, 0));
    }

    // hyst_control_var
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("hyst_control_var", ParameterParserDataType::Enum, (double)(UINT32)hyst_control_var, 0));
    }
    else if (PARAMETER_STRCMP(__name, "hyst_control_var") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Enum, (double)(UINT32)hyst_control_var, 0));
    }

    // hyst_mode
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("hyst_mode", ParameterParserDataType::Enum, (double)(UINT32)hyst_mode, 0));
    }
    else if (PARAMETER_STRCMP(__name, "hyst_mode") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Enum, (double)(UINT32)hyst_mode, 0));
    }

    // hyst_trigger
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("hyst_trigger", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "hyst_trigger") == 0)
    {
        __result = hyst_trigger.GetInfo(&__path[__pos], __info);
    }

    return __result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// @brief Read enum value
/// @param entry [in] Symbol table entry
/// @param enumValue [out] Enum value to read
/// @param alignment [in] Symbol table byte alignment
/// @return true if successful
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool chromatix_abf40Type::control_variablesStruct::ReadEnum_control_var_typeEnum(
    ParameterFileSymbolTableEntry* const entry,
    chromatix_abf40Type::control_variablesStruct::control_var_typeEnum& enumValue,
    BYTE const alignment)
{
    bool result {false};

    UINT32 uintValue {entry->ReadEnum(&result, alignment)};

    if (result)
    {
        switch (uintValue)
        {
        case static_cast<UINT32>(chromatix_abf40Type::control_variablesStruct::control_var_typeEnum::control_lux_idx):
            enumValue = chromatix_abf40Type::control_variablesStruct::control_var_typeEnum::control_lux_idx;
            result    = true;
            break;
        case static_cast<UINT32>(chromatix_abf40Type::control_variablesStruct::control_var_typeEnum::control_gain):
            enumValue = chromatix_abf40Type::control_variablesStruct::control_var_typeEnum::control_gain;
            result    = true;
            break;
        case static_cast<UINT32>(chromatix_abf40Type::control_variablesStruct::control_var_typeEnum::control_drc_gain):
            enumValue = chromatix_abf40Type::control_variablesStruct::control_var_typeEnum::control_drc_gain;
            result    = true;
            break;
        case static_cast<UINT32>(chromatix_abf40Type::control_variablesStruct::control_var_typeEnum::control_exp_time_ratio):
            enumValue = chromatix_abf40Type::control_variablesStruct::control_var_typeEnum::control_exp_time_ratio;
            result    = true;
            break;
        case static_cast<UINT32>(chromatix_abf40Type::control_variablesStruct::control_var_typeEnum::control_aec_exp_sensitivity_ratio):
            enumValue = chromatix_abf40Type::control_variablesStruct::control_var_typeEnum::control_aec_exp_sensitivity_ratio;
            result    = true;
            break;
        case static_cast<UINT32>(chromatix_abf40Type::control_variablesStruct::control_var_typeEnum::control_exp_gain_ratio):
            enumValue = chromatix_abf40Type::control_variablesStruct::control_var_typeEnum::control_exp_gain_ratio;
            result    = true;
            break;
        default:
            PARAMETER_LOG_ERROR(entry->SetManager, "Invalid value(% u) converting UINT32 to chromatix_abf40Type::control_variablesStruct::control_var_typeEnum", uintValue);
            break;
        }
    }
    return result;
}


////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Default Constructor
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
chromatix_abf40TypeClass::chromatix_abf40TypeClass(
    const CHAR* const name)
    : ParameterModule(name, MAJOR, MINOR, PATCH, nullptr)
{
    PARAMETER_INITIALIZE(SymbolTableID);
    PARAMETER_INITIALIZE(enable_section);
    PARAMETER_INITIALIZE(module_version);
    PARAMETER_INITIALIZE(parameter_version);
    PARAMETER_INITIALIZE(control_variables);
    PARAMETER_INITIALIZE(control_method);
    PARAMETER_INITIALIZE(dynamic_enable_triggers);
    PARAMETER_INITIALIZE(chromatix_abf40_reserve);
    PARAMETER_INITIALIZE(chromatix_abf40_core);
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Constructor
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
chromatix_abf40TypeClass::chromatix_abf40TypeClass(
    const CHAR* const name,
    ModuleData* const moduleData)
    : ParameterModule(name, MAJOR, MINOR, PATCH, moduleData)
{
    PARAMETER_INITIALIZE(SymbolTableID);
    PARAMETER_INITIALIZE(enable_section);
    PARAMETER_INITIALIZE(module_version);
    PARAMETER_INITIALIZE(parameter_version);
    PARAMETER_INITIALIZE(control_variables);
    PARAMETER_INITIALIZE(control_method);
    PARAMETER_INITIALIZE(dynamic_enable_triggers);
    PARAMETER_INITIALIZE(chromatix_abf40_reserve);
    PARAMETER_INITIALIZE(chromatix_abf40_core);
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Destructor
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
chromatix_abf40TypeClass::~chromatix_abf40TypeClass()
{
    if (Loaded)
    {
        UnloadModule();
    }
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// GetName
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
const CHAR* chromatix_abf40TypeClass::GetName(
    const CHAR* const type)
{
    const CHAR* name = type;
    CHAR temp[ParameterFileSymbolTableEntry::TYPE_LEN + 1];
    if (PARAMETER_STRCMP(type, ModuleData::GetType(temp, "abf40_bps")) == 0)
    {
        name = "abf40_bps";
    }
    if (PARAMETER_STRCMP(type, ModuleData::GetType(temp, "abf40_ife")) == 0)
    {
        name = "abf40_ife";
    }
    return name;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load abf40_rgn_dataType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool abf40_rgn_dataType::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(minmax_maxshft);
    PARAMETER_INITIALIZE(minmax_minshft);
    PARAMETER_INITIALIZE(minmax_offset);
    PARAMETER_INITIALIZE(minmax_bls);
    PARAMETER_INITIALIZE(blkpix_lev_tab);
    PARAMETER_INITIALIZE(noise_std_lut_tab);
    PARAMETER_INITIALIZE(curve_offset_tab);
    PARAMETER_INITIALIZE(dist_ker_tab);
    PARAMETER_INITIALIZE(edge_softness_tab);
    PARAMETER_INITIALIZE(denoise_strength_tab);
    PARAMETER_INITIALIZE(noise_prsv_anchor_tab);
    PARAMETER_INITIALIZE(noise_prsv_base_tab);
    PARAMETER_INITIALIZE(radial_edge_softness_adj_tab);
    PARAMETER_INITIALIZE(radial_noise_prsv_adj_tab);
    PARAMETER_INITIALIZE(act_fac_lut_tab);
    PARAMETER_INITIALIZE(act_fac0);
    PARAMETER_INITIALIZE(act_fac1);
    PARAMETER_INITIALIZE(act_thd0);
    PARAMETER_INITIALIZE(act_thd1);
    PARAMETER_INITIALIZE(act_smth_thd0);
    PARAMETER_INITIALIZE(act_smth_thd1);
    PARAMETER_INITIALIZE(dark_thd);
    PARAMETER_INITIALIZE(dark_fac_lut_tab);
    PARAMETER_INITIALIZE(edge_detect_thd);
    PARAMETER_INITIALIZE(edge_count_low);
    PARAMETER_INITIALIZE(edge_detect_noise_scaler);
    PARAMETER_INITIALIZE(edge_smooth_strength);
    PARAMETER_INITIALIZE(edge_smooth_scaler_tab);

    SYMBOL_TABLE_ENTRY_READ(entry, result, minmax_maxshft, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, minmax_minshft, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, minmax_offset, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, minmax_bls, alignment)
    if (result)
    {
        result = blkpix_lev_tab.Load(entry, alignment);
    }
    if (result)
    {
        result = noise_std_lut_tab.Load(entry, alignment);
    }
    if (result)
    {
        result = curve_offset_tab.Load(entry, alignment);
    }
    if (result)
    {
        result = dist_ker_tab.Load(entry, alignment);
    }
    if (result)
    {
        result = edge_softness_tab.Load(entry, alignment);
    }
    if (result)
    {
        result = denoise_strength_tab.Load(entry, alignment);
    }
    if (result)
    {
        result = noise_prsv_anchor_tab.Load(entry, alignment);
    }
    if (result)
    {
        result = noise_prsv_base_tab.Load(entry, alignment);
    }
    if (result)
    {
        result = radial_edge_softness_adj_tab.Load(entry, alignment);
    }
    if (result)
    {
        result = radial_noise_prsv_adj_tab.Load(entry, alignment);
    }
    if (result)
    {
        result = act_fac_lut_tab.Load(entry, alignment);
    }
    SYMBOL_TABLE_ENTRY_READ(entry, result, act_fac0, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, act_fac1, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, act_thd0, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, act_thd1, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, act_smth_thd0, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, act_smth_thd1, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, dark_thd, alignment)
    if (result)
    {
        result = dark_fac_lut_tab.Load(entry, alignment);
    }
    SYMBOL_TABLE_ENTRY_READ(entry, result, edge_detect_thd, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, edge_count_low, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, edge_detect_noise_scaler, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, edge_smooth_strength, alignment)
    if (result)
    {
        result = edge_smooth_scaler_tab.Load(entry, alignment);
    }

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load abf40_rgn_dataType_blkpix_lev_tabStruct
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool abf40_rgn_dataType::blkpix_lev_tabStruct::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(blkpix_lev);

    result = entry->ReadArray(blkpix_lev, 2, alignment);

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load abf40_rgn_dataType_noise_std_lut_tabStruct
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool abf40_rgn_dataType::noise_std_lut_tabStruct::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(noise_std_lut);

    result = entry->ReadArray(noise_std_lut, 65, alignment);

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load abf40_rgn_dataType_curve_offset_tabStruct
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool abf40_rgn_dataType::curve_offset_tabStruct::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(curve_offset);

    result = entry->ReadArray(curve_offset, 4, alignment);

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load abf40_rgn_dataType_dist_ker_tabStruct
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool abf40_rgn_dataType::dist_ker_tabStruct::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(dist_ker);

    result = entry->ReadArray(dist_ker, 18, alignment);

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load abf40_rgn_dataType_edge_softness_tabStruct
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool abf40_rgn_dataType::edge_softness_tabStruct::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(edge_softness);

    result = entry->ReadArray(edge_softness, 4, alignment);

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load abf40_rgn_dataType_denoise_strength_tabStruct
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool abf40_rgn_dataType::denoise_strength_tabStruct::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(denoise_strength);

    result = entry->ReadArray(denoise_strength, 4, alignment);

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load abf40_rgn_dataType_noise_prsv_anchor_tabStruct
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool abf40_rgn_dataType::noise_prsv_anchor_tabStruct::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(noise_prsv_anchor);

    result = entry->ReadArray(noise_prsv_anchor, 5, alignment);

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load abf40_rgn_dataType_noise_prsv_base_tabStruct
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool abf40_rgn_dataType::noise_prsv_base_tabStruct::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(noise_prsv_base);

    result = entry->ReadArray(noise_prsv_base, 10, alignment);

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load abf40_rgn_dataType_radial_edge_softness_adj_tabStruct
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool abf40_rgn_dataType::radial_edge_softness_adj_tabStruct::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(radial_edge_softness_adj);

    result = entry->ReadArray(radial_edge_softness_adj, 5, alignment);

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load abf40_rgn_dataType_radial_noise_prsv_adj_tabStruct
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool abf40_rgn_dataType::radial_noise_prsv_adj_tabStruct::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(radial_noise_prsv_adj);

    result = entry->ReadArray(radial_noise_prsv_adj, 5, alignment);

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load abf40_rgn_dataType_act_fac_lut_tabStruct
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool abf40_rgn_dataType::act_fac_lut_tabStruct::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(act_fac_lut);

    result = entry->ReadArray(act_fac_lut, 32, alignment);

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load abf40_rgn_dataType_dark_fac_lut_tabStruct
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool abf40_rgn_dataType::dark_fac_lut_tabStruct::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(dark_fac_lut);

    result = entry->ReadArray(dark_fac_lut, 42, alignment);

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load abf40_rgn_dataType_edge_smooth_scaler_tabStruct
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool abf40_rgn_dataType::edge_smooth_scaler_tabStruct::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(edge_smooth_scaler);

    result = entry->ReadArray(edge_smooth_scaler, 4, alignment);

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load mod_abf40_aec_dataType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool mod_abf40_aec_dataType::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(aec_trigger);
    PARAMETER_INITIALIZE(abf40_rgn_data);

    result = aec_trigger.Load(entry, alignment);
    if (result)
    {
        result = abf40_rgn_data.Load(entry, alignment);
    }

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load mod_abf40_hdr_aec_dataType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool mod_abf40_hdr_aec_dataType::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(hdr_aec_trigger);
    PARAMETER_INITIALIZE(hdr_aec_data);

    result = hdr_aec_trigger.Load(entry, alignment);
    if (result)
    {
        result = hdr_aec_data.Load(entry, alignment);
    }

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load mod_abf40_hdr_aec_dataType_hdr_aec_dataStruct
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool mod_abf40_hdr_aec_dataType::hdr_aec_dataStruct::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(mod_abf40_aec_data);


    // Reading Array mod_abf40_aec_data from symbol table
    // Do not check result value here because the count value needs to be initialized in this block
    {
        mod_abf40_aec_dataCount = 0;
        SYMBOL_TABLE_ENTRY_READ(entry, result, mod_abf40_aec_dataCount, alignment)
        constexpr const CHAR* const file {__FILE__};
        constexpr INT32 line {LINE_CONST};
        ParameterFileSymbolTableEntry  pointerEntryCls;
        ParameterFileSymbolTableEntry* const pointerEntry = &pointerEntryCls;
        if (result)
        {
            result = entry->Table->ReadPointerEntry(pointerEntryCls, entry, alignment);
        }

        if (result)
        {
            mod_abf40_aec_dataID = pointerEntry->ID;
            DupObjInfo* mod_abf40_aec_dataDupInfo = pointerEntry->GetDupObject(mod_abf40_aec_dataCount, file, line);
            if (mod_abf40_aec_dataDupInfo != nullptr && mod_abf40_aec_dataDupInfo->Object != nullptr)
            {
                mod_abf40_aec_data = (mod_abf40_aec_dataType*)mod_abf40_aec_dataDupInfo->Object;
                if (mod_abf40_aec_data == nullptr)
                {
                    if (mod_abf40_aec_dataDupInfo->File != nullptr)
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "mod_abf40_aec_data duplicate found but object is nullptr: %s:%d, RefCount: %d\n",
                            mod_abf40_aec_dataDupInfo->File, mod_abf40_aec_dataDupInfo->Line, mod_abf40_aec_dataDupInfo->RefCount);
                    }
                    else
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "mod_abf40_aec_data duplicate found but Object is nullptr: <nullptr file ptr>:%d, RefCount: %d\n",
                            mod_abf40_aec_dataDupInfo->Line, mod_abf40_aec_dataDupInfo->RefCount);
                    }
                }
            }

            if (mod_abf40_aec_data == nullptr)
            {
                if (result && mod_abf40_aec_dataCount > 0)
                {
                    mod_abf40_aec_data = PARAMETER_NEW mod_abf40_aec_dataType[mod_abf40_aec_dataCount];
                    (void)pointerEntry->SetDupObject(mod_abf40_aec_data, mod_abf40_aec_dataCount, file, line);
                }
                if (!result || mod_abf40_aec_data == nullptr)
                {
                    if (mod_abf40_aec_dataCount > 0)
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "Failed to allocate mod_abf40_aec_data\n");
                    }
                    mod_abf40_aec_dataCount = 0;
                }
                for (UINT32 i = 0; result && i < mod_abf40_aec_dataCount; i++)
                {
                    result = mod_abf40_aec_data[i].Load(pointerEntry, alignment);
                }
            }
        }
        if (mod_abf40_aec_data == nullptr)
        {
            mod_abf40_aec_dataCount = 0;
        }
    }

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load mod_abf40_drc_gain_dataType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool mod_abf40_drc_gain_dataType::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(drc_gain_trigger);
    PARAMETER_INITIALIZE(drc_gain_data);

    result = drc_gain_trigger.Load(entry, alignment);
    if (result)
    {
        result = drc_gain_data.Load(entry, alignment);
    }

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load mod_abf40_drc_gain_dataType_drc_gain_dataStruct
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool mod_abf40_drc_gain_dataType::drc_gain_dataStruct::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(mod_abf40_hdr_aec_data);


    // Reading Array mod_abf40_hdr_aec_data from symbol table
    // Do not check result value here because the count value needs to be initialized in this block
    {
        mod_abf40_hdr_aec_dataCount = 0;
        SYMBOL_TABLE_ENTRY_READ(entry, result, mod_abf40_hdr_aec_dataCount, alignment)
        constexpr const CHAR* const file {__FILE__};
        constexpr INT32 line {LINE_CONST};
        ParameterFileSymbolTableEntry  pointerEntryCls;
        ParameterFileSymbolTableEntry* const pointerEntry = &pointerEntryCls;
        if (result)
        {
            result = entry->Table->ReadPointerEntry(pointerEntryCls, entry, alignment);
        }

        if (result)
        {
            mod_abf40_hdr_aec_dataID = pointerEntry->ID;
            DupObjInfo* mod_abf40_hdr_aec_dataDupInfo = pointerEntry->GetDupObject(mod_abf40_hdr_aec_dataCount, file, line);
            if (mod_abf40_hdr_aec_dataDupInfo != nullptr && mod_abf40_hdr_aec_dataDupInfo->Object != nullptr)
            {
                mod_abf40_hdr_aec_data = (mod_abf40_hdr_aec_dataType*)mod_abf40_hdr_aec_dataDupInfo->Object;
                if (mod_abf40_hdr_aec_data == nullptr)
                {
                    if (mod_abf40_hdr_aec_dataDupInfo->File != nullptr)
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "mod_abf40_hdr_aec_data duplicate found but object is nullptr: %s:%d, RefCount: %d\n",
                            mod_abf40_hdr_aec_dataDupInfo->File, mod_abf40_hdr_aec_dataDupInfo->Line, mod_abf40_hdr_aec_dataDupInfo->RefCount);
                    }
                    else
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "mod_abf40_hdr_aec_data duplicate found but Object is nullptr: <nullptr file ptr>:%d, RefCount: %d\n",
                            mod_abf40_hdr_aec_dataDupInfo->Line, mod_abf40_hdr_aec_dataDupInfo->RefCount);
                    }
                }
            }

            if (mod_abf40_hdr_aec_data == nullptr)
            {
                if (result && mod_abf40_hdr_aec_dataCount > 0)
                {
                    mod_abf40_hdr_aec_data = PARAMETER_NEW mod_abf40_hdr_aec_dataType[mod_abf40_hdr_aec_dataCount];
                    (void)pointerEntry->SetDupObject(mod_abf40_hdr_aec_data, mod_abf40_hdr_aec_dataCount, file, line);
                }
                if (!result || mod_abf40_hdr_aec_data == nullptr)
                {
                    if (mod_abf40_hdr_aec_dataCount > 0)
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "Failed to allocate mod_abf40_hdr_aec_data\n");
                    }
                    mod_abf40_hdr_aec_dataCount = 0;
                }
                for (UINT32 i = 0; result && i < mod_abf40_hdr_aec_dataCount; i++)
                {
                    result = mod_abf40_hdr_aec_data[i].Load(pointerEntry, alignment);
                }
            }
        }
        if (mod_abf40_hdr_aec_data == nullptr)
        {
            mod_abf40_hdr_aec_dataCount = 0;
        }
    }

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load chromatix_abf40_coreType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool chromatix_abf40_coreType::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(mod_abf40_drc_gain_data);


    // Reading Array mod_abf40_drc_gain_data from symbol table
    // Do not check result value here because the count value needs to be initialized in this block
    {
        mod_abf40_drc_gain_dataCount = 0;
        SYMBOL_TABLE_ENTRY_READ(entry, result, mod_abf40_drc_gain_dataCount, alignment)
        constexpr const CHAR* const file {__FILE__};
        constexpr INT32 line {LINE_CONST};
        ParameterFileSymbolTableEntry  pointerEntryCls;
        ParameterFileSymbolTableEntry* const pointerEntry = &pointerEntryCls;
        if (result)
        {
            result = entry->Table->ReadPointerEntry(pointerEntryCls, entry, alignment);
        }

        if (result)
        {
            mod_abf40_drc_gain_dataID = pointerEntry->ID;
            DupObjInfo* mod_abf40_drc_gain_dataDupInfo = pointerEntry->GetDupObject(mod_abf40_drc_gain_dataCount, file, line);
            if (mod_abf40_drc_gain_dataDupInfo != nullptr && mod_abf40_drc_gain_dataDupInfo->Object != nullptr)
            {
                mod_abf40_drc_gain_data = (mod_abf40_drc_gain_dataType*)mod_abf40_drc_gain_dataDupInfo->Object;
                if (mod_abf40_drc_gain_data == nullptr)
                {
                    if (mod_abf40_drc_gain_dataDupInfo->File != nullptr)
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "mod_abf40_drc_gain_data duplicate found but object is nullptr: %s:%d, RefCount: %d\n",
                            mod_abf40_drc_gain_dataDupInfo->File, mod_abf40_drc_gain_dataDupInfo->Line, mod_abf40_drc_gain_dataDupInfo->RefCount);
                    }
                    else
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "mod_abf40_drc_gain_data duplicate found but Object is nullptr: <nullptr file ptr>:%d, RefCount: %d\n",
                            mod_abf40_drc_gain_dataDupInfo->Line, mod_abf40_drc_gain_dataDupInfo->RefCount);
                    }
                }
            }

            if (mod_abf40_drc_gain_data == nullptr)
            {
                if (result && mod_abf40_drc_gain_dataCount > 0)
                {
                    mod_abf40_drc_gain_data = PARAMETER_NEW mod_abf40_drc_gain_dataType[mod_abf40_drc_gain_dataCount];
                    (void)pointerEntry->SetDupObject(mod_abf40_drc_gain_data, mod_abf40_drc_gain_dataCount, file, line);
                }
                if (!result || mod_abf40_drc_gain_data == nullptr)
                {
                    if (mod_abf40_drc_gain_dataCount > 0)
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "Failed to allocate mod_abf40_drc_gain_data\n");
                    }
                    mod_abf40_drc_gain_dataCount = 0;
                }
                for (UINT32 i = 0; result && i < mod_abf40_drc_gain_dataCount; i++)
                {
                    result = mod_abf40_drc_gain_data[i].Load(pointerEntry, alignment);
                }
            }
        }
        if (mod_abf40_drc_gain_data == nullptr)
        {
            mod_abf40_drc_gain_dataCount = 0;
        }
    }

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load chromatix_abf40_reserveType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool chromatix_abf40_reserveType::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(cross_plane_en);
    PARAMETER_INITIALIZE(dark_desat_en);
    PARAMETER_INITIALIZE(dark_smooth_en);
    PARAMETER_INITIALIZE(act_adj_en);
    PARAMETER_INITIALIZE(blk_opt);
    PARAMETER_INITIALIZE(radial_anchor_tab);

    SYMBOL_TABLE_ENTRY_READ(entry, result, cross_plane_en, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, dark_desat_en, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, dark_smooth_en, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, act_adj_en, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, blk_opt, alignment)
    if (result)
    {
        result = radial_anchor_tab.Load(entry, alignment);
    }

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load chromatix_abf40_reserveType_radial_anchor_tabStruct
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool chromatix_abf40_reserveType::radial_anchor_tabStruct::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(radial_anchor);

    result = entry->ReadArray(radial_anchor, 5, alignment);

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load chromatix_abf40Type
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool chromatix_abf40Type::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(enable_section);
    PARAMETER_INITIALIZE(module_version);
    PARAMETER_INITIALIZE(parameter_version);
    PARAMETER_INITIALIZE(control_variables);
    PARAMETER_INITIALIZE(control_method);
    PARAMETER_INITIALIZE(dynamic_enable_triggers);
    PARAMETER_INITIALIZE(chromatix_abf40_reserve);
    PARAMETER_INITIALIZE(chromatix_abf40_core);

    result = enable_section.Load(entry, alignment);
    if (result)
    {
        result = module_version.Load(entry, alignment);
    }
    if (result)
    {
        result = parameter_version.Load(entry, alignment);
    }
    if (result)
    {
        result = control_variables.Load(entry, alignment);
    }
    if (result)
    {
        result = control_method.Load(entry, alignment);
    }
    if (result)
    {
        result = dynamic_enable_triggers.Load(entry, alignment);
    }
    if (result)
    {
        result = chromatix_abf40_reserve.Load(entry, alignment);
    }
    if (result)
    {
        result = chromatix_abf40_core.Load(entry, alignment);
    }

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load enable_sectionStruct
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool chromatix_abf40Type::enable_sectionStruct::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(bilateral_en);
    PARAMETER_INITIALIZE(minmax_en);
    PARAMETER_INITIALIZE(dirsmth_en);

    SYMBOL_TABLE_ENTRY_READ(entry, result, bilateral_en, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, minmax_en, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, dirsmth_en, alignment)

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load module_versionStruct
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool chromatix_abf40Type::module_versionStruct::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(major_revision);
    PARAMETER_INITIALIZE(minor_revision);
    PARAMETER_INITIALIZE(incr_revision);

    SYMBOL_TABLE_ENTRY_READ(entry, result, major_revision, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, minor_revision, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, incr_revision, alignment)

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load parameter_versionStruct
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool chromatix_abf40Type::parameter_versionStruct::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(revision);


    // Reading String revision from symbol table
    {
        UINT32 count = 0;
        SYMBOL_TABLE_ENTRY_READ(entry, result, count, alignment)
        ParameterFileSymbolTableEntry stringPointerEntryCls;
        ParameterFileSymbolTableEntry* const stringPointerEntry = &stringPointerEntryCls;
        if (result)
        {
            result = entry->Table->ReadPointerEntry(stringPointerEntryCls, entry, alignment);
        }
        SYMBOL_TABLE_ENTRY_READ_STRING(stringPointerEntry, result, revision, count, alignment)
    }

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load control_variablesStruct
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool chromatix_abf40Type::control_variablesStruct::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(control_var_type);

    for (UINT32 i = 0; result && i < 3; i++)
    {
        result = chromatix_abf40Type::control_variablesStruct::ReadEnum_control_var_typeEnum(entry, control_var_type[i], alignment);
    }

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load control_methodStruct
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool chromatix_abf40Type::control_methodStruct::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(aec_exp_control);
    PARAMETER_INITIALIZE(aec_hdr_control);

    result = ispglobalelements::ReadEnum_tuning_control_aec_type(entry, aec_exp_control, alignment);
    if (result)
    {
        result = ispglobalelements::ReadEnum_tuning_control_aec_hdr_type(entry, aec_hdr_control, alignment);
    }

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load dynamic_enable_triggersStruct
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool chromatix_abf40Type::dynamic_enable_triggersStruct::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(bilateral_en);

    result = bilateral_en.Load(entry, alignment);

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load dynamic_enable_triggersStruct_bilateral_enStruct
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool chromatix_abf40Type::dynamic_enable_triggersStruct::bilateral_enStruct::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(enable);
    PARAMETER_INITIALIZE(hyst_control_var);
    PARAMETER_INITIALIZE(hyst_mode);
    PARAMETER_INITIALIZE(hyst_trigger);

    SYMBOL_TABLE_ENTRY_READ(entry, result, enable, alignment)
    if (result)
    {
        result = ispglobalelements::ReadEnum_control_var_type(entry, hyst_control_var, alignment);
    }
    if (result)
    {
        result = ispglobalelements::ReadEnum_hyst_direction(entry, hyst_mode, alignment);
    }
    if (result)
    {
        result = hyst_trigger.Load(entry, alignment);
    }

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Unload mod_abf40_hdr_aec_dataType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void mod_abf40_hdr_aec_dataType::Unload(ParameterFileSymbolTable* const __table)
{
    hdr_aec_data.Unload(__table);
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Unload mod_abf40_hdr_aec_dataType_hdr_aec_dataStruct
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void mod_abf40_hdr_aec_dataType::hdr_aec_dataStruct::Unload(ParameterFileSymbolTable* const __table)
{
    if (__table != nullptr && __table->ReleaseDupObject(mod_abf40_aec_data))
    {
        PARAMETER_DELETE[] mod_abf40_aec_data;
    }
    mod_abf40_aec_data = nullptr;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Unload mod_abf40_drc_gain_dataType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void mod_abf40_drc_gain_dataType::Unload(ParameterFileSymbolTable* const __table)
{
    drc_gain_data.Unload(__table);
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Unload mod_abf40_drc_gain_dataType_drc_gain_dataStruct
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void mod_abf40_drc_gain_dataType::drc_gain_dataStruct::Unload(ParameterFileSymbolTable* const __table)
{
    for (UINT32 i = 0; i < mod_abf40_hdr_aec_dataCount; i++)
    {
        mod_abf40_hdr_aec_data[i].Unload(__table);
    }
    if (__table != nullptr && __table->ReleaseDupObject(mod_abf40_hdr_aec_data))
    {
        PARAMETER_DELETE[] mod_abf40_hdr_aec_data;
    }
    mod_abf40_hdr_aec_data = nullptr;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Unload chromatix_abf40_coreType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void chromatix_abf40_coreType::Unload(ParameterFileSymbolTable* const __table)
{
    for (UINT32 i = 0; i < mod_abf40_drc_gain_dataCount; i++)
    {
        mod_abf40_drc_gain_data[i].Unload(__table);
    }
    if (__table != nullptr && __table->ReleaseDupObject(mod_abf40_drc_gain_data))
    {
        PARAMETER_DELETE[] mod_abf40_drc_gain_data;
    }
    mod_abf40_drc_gain_data = nullptr;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Unload chromatix_abf40Type
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void chromatix_abf40Type::Unload(ParameterFileSymbolTable* const __table)
{
    parameter_version.Unload(__table);
    chromatix_abf40_core.Unload(__table);
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Unload parameter_versionStruct
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void chromatix_abf40Type::parameter_versionStruct::Unload(ParameterFileSymbolTable* const __table)
{
    if (__table != nullptr && __table->ReleaseDupObject(revision))
    {
        PARAMETER_DELETE[] revision;
    }
    revision = nullptr;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Release the ParameterModule object
/// Requires that this object was created via a ParameterSetManager::GetModule call
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void chromatix_abf40Type::Release()
{
    ((chromatix_abf40TypeClass*)this)->Release();
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Returns the ParameterModule::Modified value
/// The Modified value is incremented when the module's data or pointer values change
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
UINT32 chromatix_abf40Type::GetModified()
{
    return ((chromatix_abf40TypeClass*)this)->Modified;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Create module, and return a new object
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
ParameterModule* chromatix_abf40TypeClass::Create(
    ModuleData*    moduleData,
    DefaultModule* const defModule) const
{
    chromatix_abf40TypeClass* cls = nullptr;

    if (PARAMETER_STRCMP(Type, moduleData->Type) == 0 && Version.Value == moduleData->Version.Value)
    {
        const UINT32 id = moduleData->DuplicateID > 0 ? moduleData->DuplicateID : moduleData->ID;

        ParameterModule* const module = defModule->Find(id);
        if (module != nullptr)
        {
            cls = (chromatix_abf40TypeClass*)module;
        }
        else
        {
            cls = PARAMETER_NEW chromatix_abf40TypeClass(GetName(moduleData->Type), moduleData);
            cls->SymbolTableID = id;
            defModule->AddModule((ParameterModule*)cls);
        }
    }
    else if (Version.Value != moduleData->Version.Value)
    {
        PARAMETER_LOG_ERROR(moduleData->SetManager, "Unable to create chromatix_abf40TypeClass, binary version (%lld) does not match source version (%lld)\n",
            moduleData->Version.Value, Version.Value);
    }
    else
    {
        PARAMETER_LOG_ERROR(moduleData->SetManager, "Unable to create chromatix_abf40TypeClass, binary name (%s) does not match source name (%s)\n",
            moduleData->Type, Type);
    }

    return (ParameterModule*)cls;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load module data, return true is successful
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool chromatix_abf40TypeClass::LoadModule(
    ParameterFileSymbolTable* const table)
{
    bool result = true;

    if (!Loaded)
    {
        BYTE const alignment = table->Alignment;
        chromatix_abf40TypeClass* const cls = this;

        ParameterFileSymbolTableEntry  entryObj;
        ParameterFileSymbolTableEntry* const entry = &entryObj;
        // With EzTune enabled, the unused upper byte of SymbolTableID value can change, so clear it
        result = table->Find(entryObj, SymbolTableID & 0x00ffffff);

        if (!result)
        {
            PARAMETER_LOG_ERROR(entry->SetManager, "chromatix_abf40TypeClass::LoadModule Failed to find module ID (%d) in Symbol Table\n", SymbolTableID);
        }

        if (result)
        {
            result = cls->Load(entry, alignment);
        }
        Modified++;
        Loaded = true;
    }

    if (!result)
    {
        /// Module failed to fully load, unload it
        UnloadModule();
    }

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Unload module data
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void chromatix_abf40TypeClass::UnloadModule()
{
    if (Loaded)
    {
        if (Data != nullptr && Data->SymbolTable != nullptr)
        {
            Unload(Data->SymbolTable);
        }
        else
        {
            PARAMETER_LOG_ERROR(Data != nullptr ? Data->SetManager : nullptr, "UnloadModule %s called with nullptr "
                "Data->SymbolTable object\n", Name);
            Unload(nullptr);
        }
        Loaded = false;
    }
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// /// Release the ParameterModule object
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void chromatix_abf40TypeClass::Release()
{
    ParameterModule::Release();
}

ABF_4_0_0_NAMESPACE_END
